{% extends "layout.html" %}

{% block content %}
<div id="postnetviz"></div>
<div id="tweet" hidden></div>
{% endblock %}


{% block foot %}
<script type="text/javascript" src="/static/postnet.js"></script>
{% endblock %}