{% extends "layout.html" %}

{% block head %}
{% endblock %}

{% block content %}
{% endblock %}


{% block foot %}
<script type="text/javascript">
$(document).ready(function() { prompt_for_username(); });
</script>
{% endblock %}